/*
 * Vyryturvat (unfinished preview version)
 * by Heikki Kernen
 */

const int SCRIPT_GAME_VERSION = int(0x000400);

UIFont *medFont, *smallFont;
uint medFontHeight, smallFontHeight;
int screenWidth, screenHeight;

PhysicsObject *head, *neck, *trunk, *rthigh, *lthigh, *rcalf, *lcalf, *rfore, *lfore, *larm, *rarm, *rfoot, *lfoot;

int LENGTH = 30;
int travelled = 0;

uint mTime = 0;
uint treeTimer = 0;

bool finished = false;

float prevPos = 0, distance = 0;

float Z = -32.9;
float X = 44.0;

uint maxAir = 0;
uint lastTouch = 0;

uint next = 3;
bool move = false;

int treesCollected = 0;

bool treePlaced = true;

float averageSpeed = 0;
uint tick = 0;

// Called once on level initialization.
void onInit()
{
	screenWidth = getScreenWidth();
	screenHeight = getScreenHeight();
	medFont = getFont("medium");
	smallFont = getFont("small");
	medFontHeight = medFont->getFontHeight();
	smallFontHeight = smallFont->getFontHeight();

	setClearColor(0.5,0.7,0.8);

	setGravity(0.5, 0, -8.0);

	head = findObject("Head");
	neck = findObject("Neck");
	trunk = findObject("Trunk");
	rthigh = findObject("RightThigh");
	lthigh = findObject("LeftThigh");
	rarm = findObject("RightArm");
	larm = findObject("LeftArm");
	rfore = findObject("RightForearm");
	lfore = findObject("LeftForearm");
	rcalf = findObject("RightCalf");
	lcalf = findObject("LeftCalf");
	rfoot = findObject("RightFoot");
	lfoot = findObject("LeftFoot");

	initGame();

	revertColors();

}

void initGame() {
	for (uint i = 1; i<5; i++) {
		PhysicsObject *obj = getObject(i);
		obj->setColor(1,1,1);
		obj->setSize(60,25,2);
		obj->setRotation(0,0,2.5);
		obj->setPosition(0+X*float(i-1),0,-2+Z*float(i-1));
	}

	for (uint i = 5; i<9; i++) {
		PhysicsObject *obj = getObject(i);
		obj->setPosition(0+X*float(i-5),0,Z*float(i-5));
	}

	muodostaPuu(5, 5, 5 - (random()*10), -2);

}

void revertColors() {
	head->setColor(1,0.7,0.5);
	neck->setColor(1,0.7,0.5);
	trunk->setColor(1,0.7,0.5);
	rthigh->setColor(1,0.7,0.5);
	lthigh->setColor(1,0.7,0.5);
	rarm->setColor(1,0.7,0.5);
	larm->setColor(1,0.7,0.5);
	rfore->setColor(1,0.7,0.5);
	lfore->setColor(1,0.7,0.5);
	rcalf->setColor(1,0.7,0.5);
	lcalf->setColor(1,0.7,0.5);
	rfoot->setColor(1,0.7,0.5);
	lfoot->setColor(1,0.7,0.5);
}


// Called once for each game logic update before the physics update.
// Parameter 'time' is the simulation time in milliseconds.
void onUpdate(uint time)
{
	if (!finished) {
	mTime = time;
	}

	float x, y, z;
	trunk->getPosition(x, y, z);
	distance = (x > prevPos) ? x-prevPos : prevPos-x;
	prevPos = x;

	float trunkZ = z;

	getObject(next)->getPosition(x,y,z);
	if (prevPos > x) move = true;

	uint current = next - 2;
	if (current < 1 || current > 4) current += 4;

	getObject(current)->getPosition(x,y,z);
	if (trunkZ < z - 100) {
		finished = true;
	}

	if (move && travelled < LENGTH) {
		travelled++;
		PhysicsObject *col = getObject(current);

		col->getPosition(x,y,z);
		x += X*4;
		z += Z*4;
		col->setPosition(x,y,z);

		next++;

		if (next == 5) next = 1;
		move = false;
	}


	if (mTime < treeTimer) {
		puunLento(5);
	}
	else {

			if (!treePlaced) {
				PhysicsObject *obj = getObject(next);
				obj->getPosition(x,y,z);
				muodostaPuu(5, x, 5 - (random()*10), z+4);
			}

			PhysicsObject *obj = getObject(5);
			obj->getPosition(x,y,z);
			if (prevPos - x > 20) treePlaced = false;

	}

	revertColors();
}


bool onCollision(PhysicsObject *o1, PhysicsObject *o2) {

	if (o1->getType() == PHYSOBJ_RAGDOLL_BODY) o1->setColor(1,0,0);
	if (o2->getType() == PHYSOBJ_RAGDOLL_BODY) o2->setColor(1,0,0);

	uint o1id = o1->getId();
	uint o2id = o2->getId();

	if (o1->getType() == PHYSOBJ_RAGDOLL_BODY || o2->getType() == PHYSOBJ_RAGDOLL_BODY) {
		if ((o1id > 0 && o1id < 5) || (o2id > 0 && o2id < 5)) {
			uint dur = mTime - lastTouch;
			maxAir = (maxAir > dur) ? maxAir : dur;
			lastTouch = mTime;
		}

		if ((o1id > 4 && o1id < 4 + 22) || (o2id > 4 && o2id < 4 + 22)) {
			if (treeTimer < mTime) {
				treeTimer = mTime + 2000;
				treesCollected++;
			}
			return false;
		}
	}

	return true;
}

void onRender()	{
	medFont->drawString(float(screenWidth) / 2, 30, UIFONT_ALIGN_HORIZ_CENTER, "Vyryturvat preview");
	smallFont->drawString(float(screenWidth) / 2, 60, UIFONT_ALIGN_HORIZ_CENTER, "It's not where you go, but how you don't get there");

	drawSpeed();

	if (finished)
		drawVictory();
}

void drawSpeed() {
	smallFont->drawString(100,20, UIFONT_ALIGN_HORIZ_CENTER, "Speed: ");
	int speed = int(distance * 200);
	float r = 0, g = 1, b = 0;
	if (speed > 500) speed = 500;
	for (int i = 0; i < speed; i++) {
		r = float(i) / 50;
		g = 2 - r;
		float offset = float(122+i);
		smallFont->drawString(offset, 19, UIFONT_ALIGN_HORIZ_CENTER, "|", r, g, b, 1);
	}
	medFont->drawString(100,40, UIFONT_ALIGN_HORIZ_CENTER, bstrFormat(speed) + " km/h");

	if (!finished) {
		averageSpeed *= float(tick++);
		averageSpeed += float(speed);
		averageSpeed /= float(tick);
	}
}


void muodostaPuu(uint puuIndex, float x, float y, float z) {

	PhysicsObject *obj = getObject(puuIndex);

	obj->setSize(0.3, 0.3, 7.0);
	obj->setPosition(x,y,z);
	obj->setColor(0.4,0.0,0.0);

	obj = getObject(puuIndex+1);
	obj->setSize(0.2, 0.2, 1);
	obj->setColor(0.0,0.6,0.0);
	obj->setPosition(x,y,z+4);

	for (uint kerros = 0; kerros < 5; kerros++) {

		uint base = 2 + puuIndex + kerros*4;
	
		for (uint oksa = 0; oksa < 4; oksa++) {
			obj = getObject(base+oksa);
			obj->setSize(float(6-kerros), 0.1*float(6-kerros), 0.3);
			obj->setColor(0,0.6,0);
			obj->setRotation(float(oksa) / 4 * 3.14, 0 ,0);
			obj->setPosition(x,y,z + float(kerros) - 0.5);
		}

	}

	treePlaced = true;
}

void puunLento(uint puuIndex) {
	for (uint i = puuIndex; i<puuIndex+22; i++) {
		float x,y,z;
		PhysicsObject *obj = getObject(i);
		obj->getPosition(x,y,z);
		obj->setPosition(x,y,z+0.3);
	}
	treePlaced = false;
}


float random() {
	return float(mTime % 1000) / 1000.0;
}

void drawVictory()
{
	medFont->drawString(float(screenWidth) / 2,200, UIFONT_ALIGN_HORIZ_CENTER, "Average speed: " + bstrFormat(int(averageSpeed)), 1,0,0,1);
	medFont->drawString(float(screenWidth) / 2,220, UIFONT_ALIGN_HORIZ_CENTER, "Maximum air (msec): " + bstrFormat(int(maxAir)),1,0,0,1);
	medFont->drawString(float(screenWidth) / 2,240, UIFONT_ALIGN_HORIZ_CENTER, "Trees \"collected\": " + bstrFormat(treesCollected),1,0,0,1);
	medFont->drawString(float(screenWidth) / 2,260, UIFONT_ALIGN_HORIZ_CENTER, "Time taken (msec): " + bstrFormat(mTime),1,0,0,1);
}
